//-------------------------------------
//Copyright Jerome LEVEQUE 2005
//leveque_jerome@hotmail.com
//
//Part of the Sudoku game for BeOS
//-------------------------------------

#include "Window.h"
#include "Views.h"

#include <Entry.h>

#include <Alert.h>
#include <Box.h>
#include <CheckBox.h>
#include <Button.h>
#include <RadioButton.h>

//------------------------------------------------------------------

#define CASE_SIZE 40

#define SUDOKU_SIZE	(CASE_SIZE * 9 + 22)

#define MENU_HEIGHT 20.0

//------------------------------------------------------------------

int32 CaseView::show_help = FIRST_SHOW_HELP;

//------------------------------------------------------------------

SudokuView::SudokuView(BPoint _point)
			: BView(BRect(_point, BPoint(_point.x + SUDOKU_SIZE, _point.y + SUDOKU_SIZE)),
			 "SudokuView", B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS)
{
	SetViewColor(0, 0, 0);
	possibilite = new Sudoku();
}

//------------------------------------------------------------------

SudokuView::~SudokuView(void)
{
	delete possibilite;
}

//------------------------------------------------------------------

void SudokuView::AttachedToWindow(void)
{
BPoint point(1, 1);
CaseView *_cView = NULL;
	for (int32 r = 0; r < 9; r++)
	{
		for (int32 c = 0; c < 9; c++)
		{
			_cView = new CaseView(point, possibilite->GetPossibility(r, c));
			if (!_cView)
				return;
			AddChild(_cView);
			if (r + c == 0)
				_cView->MakeFocus();
			point.x += CASE_SIZE + 2;
			if (c == 2 || c == 5)
				point.x += 2;
		}
		point.x = 1;
		point.y += CASE_SIZE + 2;
		if (r == 2 || r == 5)
			point.y += 2;
	}
}

//------------------------------------------------------------------

void SudokuView::Arrow(CaseView *_case, const char _direction)
{
int32 nextview = ChildAt(_case);
	if (nextview < 0)
		return;
		
	switch (_direction)
	{
		case B_RIGHT_ARROW :
							if (((nextview + 1) % 9) == 0)
								nextview -= 8;
							else
								nextview++;
							break;
		case B_LEFT_ARROW :
							if ((nextview % 9) == 0)
								nextview += 8;
							else
								nextview--;
							break;
		case B_UP_ARROW :
							nextview -= 9;
							break;
		case B_DOWN_ARROW :
							nextview += 9;
							break;
	}
	
	if (nextview < 0)
		nextview += 81;
	if (nextview > 80)
		nextview -= 81;
		
	BView::ChildAt(nextview)->MakeFocus();

}

//------------------------------------------------------------------

void SudokuView::SetShowHelp(int32 _show_help)
{
CaseView *_caseView = ((CaseView*)BView::ChildAt(0));
	if (_caseView)
		_caseView->SetShowHelp(_show_help);
	InvalidateAllChild();
}

//------------------------------------------------------------------

void SudokuView::PrintGrid(void)
{
	if (possibilite)
		possibilite->PrintForDebugging();
}

//------------------------------------------------------------------

void SudokuView::GenerateGrid(int32 _nb_case)
{
	if (possibilite)
	{
		possibilite->SetDifficult(_nb_case);
	thread_id my_thread;
		my_thread = spawn_thread(Sudoku::Generate, "generate_thread", B_NORMAL_PRIORITY, (void *)possibilite);
		resume_thread(my_thread);
	}
}

//------------------------------------------------------------------

void SudokuView::ClearGrid(void)
{
	if (possibilite)
		possibilite->ClearAll();
	InvalidateAllChild();
}

//------------------------------------------------------------------

void SudokuView::SolveGrid(void)
{
	if (possibilite)
	{
	thread_id my_thread;
		my_thread = spawn_thread(Sudoku::Solve, "solve_thread", B_NORMAL_PRIORITY, (void *)possibilite);
		resume_thread(my_thread);
	}
}

//------------------------------------------------------------------

void SudokuView::InvalidateAllChild(void)
{
int32 i = 0;
CaseView *_caseView;
	while ((_caseView = (CaseView*)BView::ChildAt(i++)))
	{
		Looper()->Lock();
		_caseView->Invalidate();
		Looper()->Unlock();
	}

}

//------------------------------------------------------------------

void SudokuView::StopThread(void)
{
	possibilite->StopThread();
}

//------------------------------------------------------------------

void SudokuView::Undo(void)
{
	possibilite->Undo();
}

//------------------------------------------------------------------

void SudokuView::Save(void)
{
	possibilite->Save();
}

//------------------------------------------------------------------

void SudokuView::Restore(void)
{
	possibilite->Restore();
}

//------------------------------------------------------------------

int32 SudokuView::ChildAt(CaseView *_case_)
{
CaseView *thisone = NULL;
int32 i = 0;
	while ((thisone = (CaseView*)BView::ChildAt(i)))
	{
		if (thisone == _case_)
			return i;
		i++;
	}
return -1;
}

//------------------------------------------------------------------

CaseView::CaseView(BPoint _point, Possibility *_help)
		: BView(BRect(_point, BPoint(_point.x + CASE_SIZE, _point.y + CASE_SIZE)),
			"CaseView", B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE)
{
	help = _help;
	help->AddView(this);
}

//------------------------------------------------------------------

void CaseView::MouseDown(BPoint point)
{
	MakeFocus();
}

//------------------------------------------------------------------

void CaseView::KeyDown(const char *bytes, int32 numBytes)
{
	if (numBytes == 1)
	{
		if ((bytes[0] >= '1') && (bytes[0] <= '9'))
		{
			help->Set(bytes[0] - '0');
			return;
		}
		if (bytes[0] == '0')
		{
			help->Clear();
			return;
		}
		if ((bytes[0] >= B_LEFT_ARROW) && (bytes[0] <= B_DOWN_ARROW))
		{
			((SudokuView*)Parent())->Arrow(this, bytes[0]);
			return;
		}
	}
	BView::KeyDown(bytes, numBytes);
}

//------------------------------------------------------------------

void CaseView::MakeFocus(bool focused = true)
{
	Looper()->Lock();
	Invalidate();
	BView::MakeFocus(focused);
	Looper()->Unlock();
}

//------------------------------------------------------------------

void CaseView::Draw(BRect updateRect)
{
BFont font;
	GetFont(&font);
	
	if (IsFocus())
	{
		SetHighColor(255, 0, 0);
		StrokeRect(Bounds());
	}
	
	if (help->IsSet())
	{
		SetHighColor(0, 0, 0);
		font.SetSize(CASE_SIZE);
		SetFont(&font, B_FONT_SIZE | B_FONT_FLAGS);
		char c = help->Value() + '0';
		DrawChar(c, Center(c));
		return;
	}
	if (show_help == B_CONTROL_OFF)
		return;
BPoint point(CASE_SIZE / 10, CASE_SIZE / 3);
	for (char c = 1; c < 10; c++)
	{
		if (help->IsPossible(c))
		{
			SetHighColor(0, 0, 255);
			font.SetSize(CASE_SIZE / 3);
			SetFont(&font, B_FONT_SIZE | B_FONT_FLAGS);
			DrawChar(c + '0', point);
		}
			if (c == 3 || c == 6)
			{
				point.x = CASE_SIZE / 10;
				point.y += CASE_SIZE / 3;
			}
			else
				point.x += CASE_SIZE / 3;
	}
}

//------------------------------------------------------------------

void CaseView::SetShowHelp(int32 _show_help)
{
	show_help = _show_help;
}

//------------------------------------------------------------------

BPoint CaseView::Center(const char c)
{
BRect view = Bounds(), character;
	BFont font;
	GetFont(&font);
	font.GetBoundingBoxesAsString(&c, 1, B_SCREEN_METRIC, 0, &character);
BPoint result;
	result.x = (view.Width() - character.Width()) / 2.0;
	result.y = view.bottom - (view.Height() - character.Height()) / 2.0;
return result;
}

//------------------------------------------------------------------

BaseView::BaseView(BRect r) : 
		BView(r, "base_view", B_FOLLOW_ALL, B_WILL_DRAW)
{
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
}

//------------------------------------------------------------------

void BaseView::AttachedToWindow(void)
{
BBox *_bbox_;
BMessage *_mess_;
BCheckBox *_cbox_;
BButton *_button_;
BRadioButton *_radio_button_;
//-----------------
	_bbox_ = new BBox(BRect(420, 20, 530, 160));
	_bbox_->SetLabel("Help");
	_mess_ = new BMessage(SHOW_PROPOSITION);
	_cbox_ = new BCheckBox(BRect(5, 15 , 105, 35), "show_prop", "Show Proposition", _mess_);
	_bbox_->AddChild(_cbox_);
	_cbox_->SetValue(FIRST_SHOW_HELP);
	_mess_ = new BMessage(MSG_UNDO);
	_button_ = new BButton(BRect(25, 40 , 90, 60), "undo_button", "Undo", _mess_);
//	_button_->SetEnabled(false);
	_bbox_->AddChild(_button_);
	_mess_ = new BMessage(MSG_SAVE);
	_button_ = new BButton(BRect(25, 70 , 90, 90), "save_button", "Save", _mess_);
//	_button_->SetEnabled(false);
	_bbox_->AddChild(_button_);
	_mess_ = new BMessage(MSG_RESTORE);
	_button_ = new BButton(BRect(25, 100 , 90, 120), "restore_button", "Restore", _mess_);
//	_button_->SetEnabled(false);
	_bbox_->AddChild(_button_);
	AddChild(_bbox_);
//-----------------
	_mess_ = new BMessage(GENERATE_GRID);
	_bbox_ = new BBox(BRect(420, 170, 530, 270));
	_mess_->AddPointer("box_pointer", _bbox_);
	_button_ = new BButton(BRect(0, 0, 10, 10), "generate_button", "Generate a grid", _mess_);	
	_bbox_->SetLabel(_button_);
	_radio_button_ = new BRadioButton(BRect(5, 30, 50, 50), "easy_generate", "Easy", NULL);
	_radio_button_->SetValue(B_CONTROL_ON);
	_bbox_->AddChild(_radio_button_);
	_radio_button_ = new BRadioButton(BRect(5, 50, 60, 70), "medium_generate", "Medium", NULL);
	_bbox_->AddChild(_radio_button_);
	_radio_button_ = new BRadioButton(BRect(5, 70, 50, 90), "hard_generate", "Hard", NULL);
	_bbox_->AddChild(_radio_button_);
	AddChild(_bbox_);
//-----------------
	_bbox_ = new BBox(BRect(420, 280, 530, 360));
	_bbox_->SetLabel("Grid");
	_mess_ = new BMessage(CLEAR_GRID);
	_button_ = new BButton(BRect(25, 15 , 90, 35), "clear_button", "Clear", _mess_);	
	_bbox_->AddChild(_button_);
	_mess_ = new BMessage(SOLVE_GRID);
	_button_ = new BButton(BRect(25, 45 , 90, 65), "solve_button", "Solve", _mess_);	
	_bbox_->AddChild(_button_);
	AddChild(_bbox_);
//-----------------
	_mess_ = new BMessage(KILL_THREAD);
	_button_ = new BButton(BRect(445, 370 , 510, 390), "kill_button", "Stop", _mess_);	
	AddChild(_button_);
//-----------------
}

//------------------------------------------------------------------
