//-------------------------------------
//Copyright Jerome LEVEQUE 2005
//leveque_jerome@hotmail.com
//
//Part of the Sudoku game for BeOS
//-------------------------------------

#ifndef _ALL_VIEWS_H
#define _ALL_VIEWS_H

//------------------------------------------------------------------

#include "defs.h"

#include <View.h>

//------------------------------------------------------------------

#define SHOW_PROPOSITION 'shpr'

#define MSG_UNDO 'undo'
#define MSG_SAVE 'save'
#define MSG_RESTORE 'rest'

#define KILL_THREAD 'kith'

#define GENERATE_GRID 'gegr'
#define CLEAR_GRID 'clgr'
#define SOLVE_GRID 'sogr'

#define FIRST_SHOW_HELP B_CONTROL_ON

//------------------------------------------------------------------

class SudokuView : public BView
{
public:
	SudokuView(BPoint _point);
	~SudokuView(void);
	
	void AttachedToWindow(void);
	void Arrow(CaseView *_case, const char _direction);
	
	void SetShowHelp(int32 _show_help);
	void GenerateGrid(int32 _nb_case);
	void PrintGrid(void);
	void ClearGrid(void);
	void SolveGrid(void);
	void InvalidateAllChild(void);
	void StopThread(void);
	
	void Undo(void);
	void Save(void);
	void Restore(void);
	
private:
	int32 ChildAt(CaseView *_case_);

	Sudoku *possibilite;
	
};

//------------------------------------------------------------------

class CaseView : public BView
{
public:
	CaseView(BPoint _point, Possibility *_help);
	
	void MouseDown(BPoint point);
	void KeyDown(const char *bytes, int32 numBytes);
	
	virtual void MakeFocus(bool focused = true);
	virtual void Draw(BRect updateRect);
	
	static void SetShowHelp(int32 _show_help);
	
private:
	BPoint Center(char c);

	Possibility *help;
	static int32 show_help;
};

//------------------------------------------------------------------

class BaseView : public BView
{
public:
	BaseView(BRect r);
	void AttachedToWindow(void);
};

//------------------------------------------------------------------

#endif
