//-------------------------------------
//Copyright Jerome LEVEQUE 2005
//leveque_jerome@hotmail.com
//
//Part of the Sudoku game for BeOS
//-------------------------------------

#include "Window.h"
#include "Views.h"

#include <Alert.h>
#include <RadioButton.h>

//------------------------------------------------------------------

BaseWindow::BaseWindow(BRect size)
			: BWindow(size, "Sudoku", B_TITLED_WINDOW, B_NOT_RESIZABLE)
{
	sudoku_view = new SudokuView(BPoint(20, 20));
	Lock();
BaseView *v = new BaseView(Bounds());
	v->AddChild(sudoku_view);
	AddChild(v);
	Unlock();
}

//------------------------------------------------------------------

BaseWindow::~BaseWindow(void)
{
	KillThread();
	Lock();
BaseView *v = (BaseView*)ChildAt(0);
	RemoveChild(v);
	Unlock();
	delete v;
}

//------------------------------------------------------------------

void BaseWindow::MessageReceived(BMessage *msg)
{
int32 controlvalue = 0;
BView *_box_view_ = NULL;
	switch (msg->what)
	{
		case SHOW_PROPOSITION :
			if (msg->FindInt32("be:value", &controlvalue) == B_OK)
				sudoku_view->SetShowHelp(controlvalue);
			break;		
		case KILL_THREAD :
			sudoku_view->StopThread();
			break;
		case GENERATE_GRID :
			if (msg->FindPointer("box_pointer", (void**)&_box_view_) == B_OK)
			{
				BRadioButton *box_test = (BRadioButton*)_box_view_->FindView("easy_generate");
					if (box_test && (box_test->Value() == B_CONTROL_ON))
					{
						sudoku_view->GenerateGrid(EASY_NUMBER);
						break;
					}
					box_test = (BRadioButton*)_box_view_->FindView("medium_generate");
					if (box_test && (box_test->Value() == B_CONTROL_ON))
					{
						sudoku_view->GenerateGrid(MEDIUM_NUMBER);
						break;
					}
					box_test = (BRadioButton*)_box_view_->FindView("hard_generate");
					if (box_test && (box_test->Value() == B_CONTROL_ON))
					{
						sudoku_view->GenerateGrid(HARD_NUMBER);
						break;
					}
			}
			break;
		case CLEAR_GRID :
			sudoku_view->ClearGrid();
			break;
		case SOLVE_GRID :
			sudoku_view->SolveGrid();
			break;
		case UPDATE_NOW :
			sudoku_view->InvalidateAllChild();
			break;
		case MSG_UNDO :
			sudoku_view->Undo();
			break;
		case MSG_SAVE :
			sudoku_view->Save();
			break;
		case MSG_RESTORE :
			sudoku_view->Restore();
			break;
		default : BWindow::MessageReceived(msg);
	}
}

//------------------------------------------------------------------

bool BaseWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

//------------------------------------------------------------------

void BaseWindow::KillThread(void)
{
thread_id generate_thread = find_thread("generate_thread");
	if (generate_thread != B_NAME_NOT_FOUND)
		kill_thread(generate_thread);
thread_id solve_thread = find_thread("solve_thread");
	if (solve_thread != B_NAME_NOT_FOUND)
		kill_thread(solve_thread);
}

//------------------------------------------------------------------
