//-------------------------------------
//Copyright Jerome LEVEQUE 2005
//leveque_jerome@hotmail.com
//
//Part of the Sudoku game for BeOS
//-------------------------------------

#include "defs.h"
#include "Views.h"
#include "App.h"

#include <printf.h>
#include <iostream.h>
#include <stdio.h>
#include <Alert.h>
#include <stdlib.h>

//----------------------------------------------
//----------------------------------------------

#define __LOOP_ROW_COLUMN__ for (int32 r = 0; r < 9; r++) for (int32 c = 0; c < 9; c++)

//----------------------------------------------

int32 Sudoku::stopthread = false;

//----------------------------------------------

Possibility::Possibility(Sudoku *_grid)
{
	grid = _grid;
	view = NULL;
	poss = 0x1FF;
	value = NOT_YET_DEFINED;
	set = false;
}

//----------------------------------------------

Possibility::Possibility(Sudoku *_newgrid, Possibility *_tocopy)
{
	grid = _newgrid;
	view = NULL;
	poss = _tocopy->poss;
	value = _tocopy->value;
	set = _tocopy->set;
}

//----------------------------------------------
/*
Possibility::~Possibility(void)
{
}
*/
//----------------------------------------------

void Possibility::UpdateData(Possibility *_new_data)
{
	if (_new_data != NULL)
	{
		poss = _new_data->poss;
		value = _new_data->value;
		set = _new_data->set;
	}
}

//----------------------------------------------

int32 Possibility::Remove(uint8 _nb)
{
	if ((_nb < 1) || (_nb > 9))
		return -1;
	UpdateView();
int8 precount = NumberOfPossibility(), postcount;
uint16 mask = 0x1 << (_nb - 1);
	poss &= ~mask;
	postcount = NumberOfPossibility();
	if (precount != postcount)
		return postcount;
	else
		return ALREADY_REMOVED;
}

//----------------------------------------------

int32 Possibility::Add(uint8 _nb)
{
	if ((_nb < 1) || (_nb > 9))
		return -1;
	UpdateView();
int8 precount = NumberOfPossibility(), postcount;
uint16 mask = 0x1 << (_nb - 1);
	poss |= mask;
	postcount = NumberOfPossibility();
	if (precount != postcount)
		return postcount;
	else
		return ALREADY_SET;
}

//----------------------------------------------

int32 Possibility::Set(uint8 _nb, bool from_undo = false)
{
	if (set)
		return ALREADY_SET;
	if (!IsPossible(_nb))
		return ALREADY_REMOVED;
	if (_nb == 0)
		return 0;
	value = _nb;
	set = true;
	if (grid != NULL)
	{
		grid->Update_row_column_block(this, _nb, REMOVE);
		if (!from_undo)
			grid->Add(this, _nb, SET);
	}
	return _nb;
}

//----------------------------------------------

void Possibility::Clear(bool _quick, bool from_undo = false)
{
	if ((grid != NULL) && (!from_undo))
			grid->Add(this, Value(), CLEAR);
	if (_quick)
	{
		set = false;
		value = NOT_YET_DEFINED;
		poss = 0x1FF;
	}
	else
	{
	int32 oldval = Value();
		set = false;
		value = NOT_YET_DEFINED;
		if ((grid != NULL) && (oldval != NOT_YET_DEFINED))
		{
			grid->Update_row_column_block(this, oldval, ADD);
			grid->CheckAllGrid();
		}
	}

}

//----------------------------------------------

void Possibility::ClearPossibility(void)
{
	poss = 0x1FF;
}

//----------------------------------------------

int32 Possibility::Value(void)
{
	if (set)
		return value;
	else
		return NOT_YET_DEFINED;
}

//----------------------------------------------

int32 Possibility::NumberOfPossibility(void)
{
int8 result = 0;
uint16 temp = poss;
	for (int16 i = 1; i < 10; i++)
	{		
		if (temp & 0x1)
			result++;
		temp >>= 1;
	}
return result;
}

//----------------------------------------------

uint16 Possibility::GetPossibility(void)
{
	return poss;
}

//----------------------------------------------

uint8 Possibility::GetPossibility(int32 _nb)
{
uint16 temp = poss;
int32 _first = 0;
	for (int16 i = 1; i < 10; i++)
	{		
		if (temp & 0x1)
		{
			_first++;
			if (_nb == _first)
				return i;
		}
		temp >>= 1;
	}
return 0;
}

//----------------------------------------------

bool Possibility::IsSet(void)
{
	return set;
}

//----------------------------------------------

bool Possibility::IsPossible(uint8 test)
{
	if (test < 1 || test > 9)
		return false;

uint16 temp = 0x001 << (test - 1);

	if ((temp & poss) != 0)
		return true;
	else
		return false;
}

//----------------------------------------------

char *Possibility::ToString(char *_text, bool to_stdout = true)
{
char *debut = _text;
int16 temp = poss;
	for (int16 i = 1; i < 10; i++)
	{		
		if (temp & 0x1)
			*_text++ = (i + '0');
		else
			*_text++ = '.';
		temp >>= 1;
	}
	*_text = 0;
	if (to_stdout)
		printf("%s\n", debut);
return debut;
}

//----------------------------------------------

void Possibility::UpdateView(void)
{
	if (view)
	{
		view->Looper()->Lock();
		view->Invalidate();
		view->Looper()->Unlock();
	}
}

//----------------------------------------------

void Possibility::AddView(CaseView *_view)
{
	view = _view;
}

//----------------------------------------------

Sudoku::Sudoku(void)
{
	AllDefined = true;
	difficult = 0;
	undo = new BList();
	__LOOP_ROW_COLUMN__
		if ((sudoku[r][c] = new Possibility(this)) == NULL)
			AllDefined = false;
}

//----------------------------------------------

Sudoku::Sudoku(Sudoku *_tocopy_)
{
	AllDefined = true;
	difficult = 0;
	undo = new BList();
	__LOOP_ROW_COLUMN__
	{
		if ((sudoku[r][c] = new Possibility(this, _tocopy_->GetPossibility(r, c))) == NULL)
			AllDefined = false;
	}
}

//----------------------------------------------

Sudoku::~Sudoku(void)
{
	Free();
	delete undo;
	__LOOP_ROW_COLUMN__
		delete sudoku[r][c];
}

//----------------------------------------------

void Sudoku::Set(uint8 _row, uint8 _col, uint16 _nb)
{
	sudoku[_row][_col]->Set(_nb);
}

//----------------------------------------------

void Sudoku::ClearAll(void)
{
	__LOOP_ROW_COLUMN__
		if (sudoku[r][c] != NULL)
			sudoku[r][c]->Clear(true);
}

//----------------------------------------------

Possibility *Sudoku::GetPossibility(uint8 _row, uint8 _col)
{
	return sudoku[_row][_col];
}

//----------------------------------------------

int32 Sudoku::GetNumberOfSolution(void)
{
	return 0;
}

//----------------------------------------------

void Sudoku::SetDifficult(int32 _difficult)
{
	difficult = _difficult;
}

//----------------------------------------------

bool Sudoku::IsStillSolvable(void)
{
	__LOOP_ROW_COLUMN__
	{
		Possibility *_poss = sudoku[r][c];
		if (!_poss->IsSet() && (_poss->NumberOfPossibility() < 1))
			return false;
	}
return true;
}

//----------------------------------------------

bool Sudoku::IsResolved(void)
{
	__LOOP_ROW_COLUMN__
	{
		Possibility *_poss = sudoku[r][c];
		if (!_poss->IsSet())
			return false;
	}
return true;
}

//----------------------------------------------

void Sudoku::PrintToStream(void)
{
char temp[10];
	for (int32 r = 0; r < 9; r++)
	{
		for (int32 c = 0; c < 9; c++)
		{
			printf("%s ", sudoku[r][c]->ToString(temp, false));
		}
		printf("\n");
	}
printf("\n\n");
}

//----------------------------------------------

void Sudoku::PrintToStreamValue(void)
{
	for (int32 r = 0; r < 9; r++)
	{
		for (int32 c = 0; c < 9; c++)
			printf("%ld", sudoku[r][c]->Value());
		printf("\n");
	}
printf("\n\n-------------------------\n\n");
}

//----------------------------------------------

void Sudoku::PrintForDebugging(void)
{
	__LOOP_ROW_COLUMN__
	{
		if (sudoku[r][c]->IsSet())
			printf("	possibilite->Set(%ld, %ld, %ld);\n", r, c, sudoku[r][c]->Value());
	}
printf("\n\n-------------------------\n\n");
}

//----------------------------------------------

void Sudoku::Update_row_column_block(Possibility *poss, int32 _val_, ADD_REMOVE _add_)
{
	__LOOP_ROW_COLUMN__
		if (sudoku[r][c] == poss)
		{
			Update_row_column_block(r, c, _val_, _add_);
			return;
		}
}

//----------------------------------------------

void Sudoku::CheckAllGrid(void)
{
Possibility *_poss_ = NULL;
	__LOOP_ROW_COLUMN__
	{
	_poss_ = sudoku[r][c];
		if (_poss_)
			_poss_->ClearPossibility();
	}
	__LOOP_ROW_COLUMN__
	{
		_poss_ = sudoku[r][c];
		int32 _val;
		if (_poss_ && ((_val = _poss_->Value()) != NOT_YET_DEFINED))
			Update_row_column_block(r, c, _val, REMOVE);
	}
}

//----------------------------------------------

int32 Sudoku::Undo(void)
{
msg_list *_lst = (msg_list*)undo->RemoveItem(undo->CountItems() - 1);
	if (_lst == NULL)
		return END_OF_LIST;
	switch (_lst->_scn)
	{
		case SET :
			if (_lst->_poss != NULL)
				_lst->_poss->Clear(false, true);
			delete _lst;
			return SET;
		case CLEAR :
			if (_lst->_poss != NULL)
				_lst->_poss->Set(_lst->_val, true);
			delete _lst;
			return CLEAR;
		case SAVE :
			delete _lst;
			return FIND_SAVE;
		default:
			return END_OF_LIST;
	}
}

//----------------------------------------------

void Sudoku::Save(void)
{
msg_list *_lst = new msg_list;
	_lst->_poss = NULL;
	_lst->_val = 0;
	_lst->_scn = SAVE;
	
	undo->AddItem(_lst);
}

//----------------------------------------------

void Sudoku::Restore(void)
{
int32 result = 0;
	do
	{
		result = Undo();
	}
	while ((result != FIND_SAVE) && (result != END_OF_LIST));
}

//----------------------------------------------

void Sudoku::Free(void)
{
msg_list *_lst =NULL;
	while ((_lst = (msg_list*)(undo->RemoveItem((int32)0))) != NULL)
	{
		delete _lst;
	}
}

//----------------------------------------------

void Sudoku::Add(Possibility *poss, int32 _val_, SET_CLEAR_SAVE _scn_)
{
msg_list *_lst = new msg_list;
	_lst->_poss = poss;
	_lst->_val = _val_;
	_lst->_scn = _scn_;
	
	undo->AddItem(_lst);
}

//----------------------------------------------

void Sudoku::StopThread(void)
{
	stopthread = true;
	be_app_messenger.SendMessage(UPDATE_NOW);
}

//----------------------------------------------

int32 Sudoku::Generate(void *data)
{
Sudoku *sud = (Sudoku*)data;
	sud->stopthread = false;
	sud->_Generate();
return B_OK;
}

//----------------------------------------------

int32 Sudoku::Solve(void *data)
{
Sudoku *sud = (Sudoku*)data;
	sud->stopthread = false;
int32 controlvalue = sud->_Solve();
	be_app_messenger.SendMessage(UPDATE_NOW);
	if (controlvalue != B_OK)
//		(new BAlert(NULL, "Solved", "Close"))->Go();
//	else
		(new BAlert(NULL, "Sorry not Solvable", "Close"))->Go();
return controlvalue;
}

//----------------------------------------------

void Sudoku::Replace(Sudoku *_new)
{
	__LOOP_ROW_COLUMN__
	{
		sudoku[r][c]->UpdateData(_new->sudoku[r][c]);
	}
}

//----------------------------------------------

void Sudoku::Update_row_column_block(uint8 _row, uint8 _col, int32 _val_, ADD_REMOVE _add_)
{
//ROW
	for (int8 i = 0; i < 9; i++)
	{
		if (_add_ == REMOVE)
			sudoku[i][_col]->Remove(_val_);
		else
			sudoku[i][_col]->Add(_val_);
	}
//COLUMN				
	for (int8 i = 0; i < 9; i++)
	{
		if (_add_ == REMOVE)
			sudoku[_row][i]->Remove(_val_);
		else
			sudoku[_row][i]->Add(_val_);
	}

//BLOCK			
	uint8 bl_r = _row / 3, bl_c = _col / 3;
	for (int8 c = 0; c < 3; c++)
	{
	int8 col = bl_c * 3 + c;
		for (int8 r = 0; r < 3; r++)
		{
		int8 row = bl_r * 3 + r;
			if (_add_ == REMOVE)
				sudoku[row][col]->Remove(_val_);
			else
				sudoku[row][col]->Add(_val_);
		}
	}
}

//----------------------------------------------

bool Sudoku::Remove1PossIn1Case(void)
{//Only 1 possibility in 1 case?
bool _cont = false;
Possibility *_poss;
	__LOOP_ROW_COLUMN__
	{
		_poss = sudoku[r][c];
		if (!_poss->IsSet() && (_poss->NumberOfPossibility() == 1))
		{
			_poss->Set(_poss->GetPossibility(1));
			_cont = true;
		}
	}
return _cont;
}

//----------------------------------------------

bool Sudoku::Remove1PossInRow(void)
{//Only 1 possibility in 1 Row?
bool _cont = false;
Possibility *_poss;
uint16 tocount;
	for (int32 r = 0; r < 9; r++)
	{//For Each Row
	uint8 count[9] = {0, 0, 0, 0, 0, 0, 0, 0, 0};
		for (int32 c = 0; c < 9; c++)
		{
			_poss = sudoku[r][c];
			if (_poss->IsSet())
				continue;
			tocount = _poss->GetPossibility();
			uint16 temp = 0x001;
			for (int32 i = 0; i < 9; i++)
			{
				if (temp & tocount)
					count[i]++;
				temp <<= 1;
			}
		}

		for (int32 i = 0; i < 9; i++)
		{
			if (count[i] == 1)
			{//A possibility is only once
				for (int32 c = 0; c < 9; c++)
				{
					_poss = sudoku[r][c];
					if (!_poss->IsSet() && _poss->IsPossible(i + 1))
					{
						_poss->Set(i + 1);
						_cont = true;
					}
				}
			}
		}
	}
return _cont;
}

//----------------------------------------------

bool Sudoku::Remove1PossInCol(void)
{//Only 1 possibility in 1 Col?
bool _cont = false;
Possibility *_poss;
uint16 tocount;
	for (int32 c = 0; c < 9; c++)
	{//For Each Row
	uint8 count[9] = {0, 0, 0, 0, 0, 0, 0, 0, 0};
		for (int32 r = 0; r < 9; r++)
		{
			_poss = sudoku[r][c];
			if (_poss->IsSet())
				continue;
			tocount = _poss->GetPossibility();
			uint16 temp = 0x001;
			for (int32 i = 0; i < 9; i++)
			{
				if (temp & tocount)
					count[i]++;
				temp <<= 1;
			}
		}

		for (int32 i = 0; i < 9; i++)
		{
			if (count[i] == 1)
			{//A possibility is only once
				for (int32 r = 0; r < 9; r++)
				{
					_poss = sudoku[r][c];
					if (!_poss->IsSet() && _poss->IsPossible(i + 1))
					{
						_poss->Set(i + 1);
						_cont = true;
					}
				}
			}
		}
	}
return _cont;
}

//----------------------------------------------

bool Sudoku::Remove1PossInBlock(void)
{//Only 1 possibility in 1 Col?
bool _cont = false;
Possibility *_poss;
uint16 tocount;

//For Each Block
	for (int32 block_r = 0; block_r < 9; block_r += 3)
		for (int32 block_c = 0; block_c < 9; block_c += 3)
		{//In the Block
			uint8 count[9] = {0, 0, 0, 0, 0, 0, 0, 0, 0};
			for (int32 r = 0; r < 3; r++)
				for (int32 c = 0; c < 3; c++)
				{
					_poss = sudoku[block_r + r][block_c + c];
					if (_poss->IsSet())
						continue;
					tocount = _poss->GetPossibility();
					uint16 temp = 0x001;
					for (int32 i = 0; i < 9; i++)
					{
						if (temp & tocount)
							count[i]++;
						temp <<= 1;
					}
				}
		
			for (int32 i = 0; i < 9; i++)
			{
				if (count[i] == 1)
				{//A possibility is only once
					for (int32 r = 0; r < 3; r++)
						for (int32 c = 0; c < 3; c++)
						{
							_poss = sudoku[block_r + r][block_c + c];
							if (!_poss->IsSet() && _poss->IsPossible(i + 1))
							{
								_poss->Set(i + 1);
								_cont = true;
							}
						}
				}
			}
		}		
return _cont;
}

//----------------------------------------------

bool Sudoku::LastChance(void)
{
Sudoku *_test = NULL;
	__LOOP_ROW_COLUMN__
	{
	Possibility *_poss = sudoku[r][c];
		if (!_poss->IsSet())
		{
		int32 _nb_poss = _poss->NumberOfPossibility();
			for (int32 i = 0; i <_nb_poss; i++)
			{
				delete _test;
				_test = new Sudoku(this);
				if (!_test->AllDefined)
				{
					delete _test;
					return false;
				}
				_test->Set(r, c, _poss->GetPossibility(i + 1));
				if (_test->_Solve() == B_OK)
				{//Victory
					Replace(_test);
//					PrintToStreamValue();
					delete _test;
					return true;
				}
			}
			return false;
		}
	}
return false;
}

//----------------------------------------------

void Sudoku::_Generate(void)
{
	Sudoku *_test = NULL;
	srand(real_time_clock());
	do
	{
		ClearAll();
		delete _test;
		if (stopthread)
			return;
		for (int32 i = 0; i < difficult; i++)
		{
		int32 new_rand = (int32)((double(rand()) / RAND_MAX) * 81);
		uint8 r = new_rand / 9, c = new_rand % 9;
			Possibility *temp = sudoku[r][c];
			if (temp == NULL)
				return;
			if (temp->IsSet())
				i--;
			else
			{
			int32 nb_poss = temp->NumberOfPossibility();
			int32 rand_poss = (int32)((double(rand()) / RAND_MAX) * nb_poss) + 1;
				temp->Set(temp->GetPossibility(rand_poss));
			}
		}
		_test = new Sudoku(this);
	}
	while(_test->_Solve() == B_ERROR);
	delete _test;
}

//----------------------------------------------

int32 Sudoku::_Solve(void)
{
	if (!AllDefined)
		return B_ERROR;

bool cont = false, cont_global = false;
	do
	{
		if (stopthread)
			return B_ERROR;
		cont_global = false;
		while (Remove1PossIn1Case())
		{
			cont_global = true;
			if (!IsStillSolvable())
				return B_ERROR;
		}
		do
		{
			cont = false;
			cont |= Remove1PossInRow();
			cont |= Remove1PossInCol();
			cont |= Remove1PossInBlock();

			if (!IsStillSolvable())
				return B_ERROR;
			cont_global |= cont;
		}
		while (cont);
	}
	while (cont_global);
		
	if (IsResolved())
	{
		Free();
		return B_OK;
	}

bool result = LastChance();
	Free();
	if (result)
		return B_OK;
	else
		return B_ERROR;
}

//----------------------------------------------
