//-------------------------------------
//Copyright Jerome LEVEQUE 2005
//leveque_jerome@hotmail.com
//
//Part of the Sudoku game for BeOS
//-------------------------------------

#ifndef __DEFS_H__
#define __DEFS_H__

//------------------------------------------------------------------

//#include <SupportDefs.h>
#include <List.h>

//----------------------------------------------

#define ALREADY_REMOVED	'alre'
#define ALREADY_SET		'alse'
#define UPDATE_NOW		'upno'

#define NOT_YET_DEFINED 0

#define EASY_NUMBER 35
#define MEDIUM_NUMBER 30
#define HARD_NUMBER 25

//----------------------------------------------

//#define SAVE			'save'

#define FIND_SAVE		'fisa'
#define END_OF_LIST		'eoli'

//----------------------------------------------

class Possibility;
class Sudoku;
class CaseView;

//----------------------------------------------

enum ADD_REMOVE {ADD, REMOVE};
enum SET_CLEAR_SAVE {SET, CLEAR, SAVE};

//----------------------------------------------

typedef struct _msg_list
	{
		Possibility *_poss;
		int32 _val;
		SET_CLEAR_SAVE _scn;
	} msg_list;


//----------------------------------------------

class Possibility
{
friend class CaseView;

public:
	Possibility(Sudoku *_grid);
	Possibility(Sudoku *_newgrid, Possibility *_tocopy);
//	~Possibility(void);

	void UpdateData(Possibility *_new_data);
	
	int32 Remove(uint8 _nb);
	int32 Add(uint8 _nb);
	
	int32 Set(uint8 _nb, bool from_undo = false);//
	void Clear(bool quick = false, bool from_undo = false);//
	void ClearPossibility(void);

	int32 Value(void);
	int32 NumberOfPossibility(void);

	uint16 GetPossibility(void); //return poss
	uint8 GetPossibility(int32 _nb); //return the _nb first possibility 1 <= _nb <= NumberOfPossibility()
	
	bool IsSet(void);
	bool IsPossible(uint8 test);
		
	char *ToString(char *_text, bool to_stdout = true);
	
private:
	void UpdateView(void);
	void AddView(CaseView *_view);
	
	Sudoku *grid;
	CaseView *view;
	uint16 poss;
	uint16 value;
	bool set;

};

//----------------------------------------------

class Sudoku
{
public:
	Sudoku(void);
	Sudoku(Sudoku *_tocopy_);
	~Sudoku(void);
	
	void Set(uint8 _row, uint8 _col, uint16 _nb);
	void ClearAll(void);
	
	Possibility *GetPossibility(uint8 _row, uint8 _col);
	int32 GetNumberOfSolution(void);
	
	void SetDifficult(int32 _difficult);
		
	bool IsStillSolvable(void);
	bool IsResolved(void);
	
	void PrintToStream(void);
	void PrintToStreamValue(void);
	void PrintForDebugging(void);
	void Update_row_column_block(Possibility *poss, int32 _val_, ADD_REMOVE _scn_);
	void CheckAllGrid(void);

//For Undo feature-----------
	int32 Undo(void);
	void Save(void);
	void Restore(void);//Call undo until FIND_SAVE
	void Free(void);
	
	void Add(Possibility *poss, int32 _val_, SET_CLEAR_SAVE _add_);
//---------------------------

//For new Thread-------------
	void StopThread(void);

	static int32 Generate(void *data);
	static int32 Solve(void *data);
//---------------------------

private:
	void Replace(Sudoku *_new);
	void Update_row_column_block(uint8 _row, uint8 _col, int32 _val_, ADD_REMOVE _add_);
	
//For Solving
	bool Remove1PossIn1Case(void);
	bool Remove1PossInRow(void);
	bool Remove1PossInCol(void);
	bool Remove1PossInBlock(void);
//Last chance for solving
	bool LastChance(void);

//---------------------------
	void _Generate(void);
	int32 _Solve(void);
//---------------------------

	Possibility *sudoku[9][9];
	bool AllDefined;
	int32 difficult;
static int32 stopthread;

	BList *undo;
};

//---------------------------

#endif
